from django.test import TestCase

from hurumap.data.utils import get_session, _engine
from hurumap.data.tables import DATA_TABLES, FieldTable


class HurumapTestCase(TestCase):
    def setUp(self):
        self.s = get_session()
        DATA_TABLES.clear()

    def field_table(self, fields, data_str):
        table = FieldTable(fields)
        self.load_data(table, data_str)

    def load_data(self, table, data_str):
        model = table.get_model('lev')

        for row in data_str.strip().split("\n"):
            parts = row.strip().split(",")
            entry = model(geo_level=parts[0], geo_code=parts[1])

            # set fields
            for i, field in enumerate(table.fields):
                setattr(entry, field, parts[2 + i])

            v = parts[-1]
            v = None if v == '' else int(v)
            entry.total = v

            self.s.add(entry)

        self.s.flush()

    def tearDown(self):
        self.s.close()
        _engine.dispose()



from importlib import import_module
import logging
import os.path

from django.apps import AppConfig, apps as django_apps


class HurumapConfig(AppConfig):
    name = 'hurumap'
    verbose_name = 'HURUmap'
    log = logging.getLogger(__name__)

    def ready(self):
        self.check_gdal()
        self.load_tables()

    def check_gdal(self):
        # GDAL is difficult to install, so we make it an optional dependency.
        # Here, we check if it's installed and warn if it isn't.
        from hurumap.geo import HAS_GDAL, gdal_missing
        if HAS_GDAL:
            import osgeo.gdal
            self.log.info("HURUmap found GDAL version %s" % (osgeo.gdal.VersionInfo("RELEASE_NAME")))
        else:
            gdal_missing()

    def load_tables(self):
        """ Search installed apps for tables.py and import it.
        This gives child HURUmap applications a chance to declare
        their tables.
        """
        for app in django_apps.get_app_configs():
            if (os.path.exists(os.path.join(app.path, 'tables.py')) or
                    os.path.exists(os.path.join(app.path, 'tables/__init__.py'))):
                import_module(app.name + '.tables')



"""
Template filters to partition lists into rows or columns.

A common use-case is for splitting a list into a table with columns::

    {% load partition %}
    <table>
    {% for row in mylist|columns:3 %}
        <tr>
        {% for item in row %}
            <td>{{ item }}</td>
        {% endfor %}
        </tr>
    {% endfor %}
    </table>
"""

from django.template import Library

register = Library()

def rows(thelist, n):
    """
    Break a list into ``n`` rows, filling up each row to the maximum equal
    length possible. For example::

        >>> l = range(10)

        >>> rows(l, 2)
        [[0, 1, 2, 3, 4], [5, 6, 7, 8, 9]]

        >>> rows(l, 3)
        [[0, 1, 2, 3], [4, 5, 6, 7], [8, 9]]

        >>> rows(l, 4)
        [[0, 1, 2], [3, 4, 5], [6, 7, 8], [9]]

        >>> rows(l, 5)
        [[0, 1], [2, 3], [4, 5], [6, 7], [8, 9]]

        >>> rows(l, 9)
        [[0, 1], [2, 3], [4, 5], [6, 7], [8, 9], [], [], [], []]

        # This filter will always return `n` rows, even if some are empty:
        >>> rows(range(2), 3)
        [[0], [1], []]
    """
    try:
        n = int(n)
        thelist = list(thelist)
    except (ValueError, TypeError):
        return [thelist]
    list_len = len(thelist)
    split = list_len // n

    if list_len % n != 0:
        split += 1
    return [thelist[split*i:split*(i+1)] for i in range(n)]

def rows_distributed(thelist, n):
    """
    Break a list into ``n`` rows, distributing columns as evenly as possible
    across the rows. For example::

        >>> l = range(10)

        >>> rows_distributed(l, 2)
        [[0, 1, 2, 3, 4], [5, 6, 7, 8, 9]]

        >>> rows_distributed(l, 3)
        [[0, 1, 2, 3], [4, 5, 6], [7, 8, 9]]

        >>> rows_distributed(l, 4)
        [[0, 1, 2], [3, 4, 5], [6, 7], [8, 9]]

        >>> rows_distributed(l, 5)
        [[0, 1], [2, 3], [4, 5], [6, 7], [8, 9]]

        >>> rows_distributed(l, 9)
        [[0, 1], [2], [3], [4], [5], [6], [7], [8], [9]]

        # This filter will always return `n` rows, even if some are empty:
        >>> rows(range(2), 3)
        [[0], [1], []]
    """
    try:
        n = int(n)
        thelist = list(thelist)
    except (ValueError, TypeError):
        return [thelist]
    list_len = len(thelist)
    split = list_len // n

    remainder = list_len % n
    offset = 0
    rows = []
    for i in range(n):
        if remainder:
            start, end = (split+1)*i, (split+1)*(i+1)
        else:
            start, end = split*i+offset, split*(i+1)+offset
        rows.append(thelist[start:end])
        if remainder:
            remainder -= 1
            offset += 1
    return rows

def columns_distributed(thelist, n):
    """
    Break a list into ``n`` columns, filling up each column to the maximum equal
    length possible. For example:

        >>> from pprint import pprint
        >>> for i in range(7, 11):
        ...     print '%sx%s:' % (i, 3)
        ...     pprint(columns(range(i), 3), width=20)
        7x3:
        [[0, 3, 6],
         [1, 4],
         [2, 5]]
        8x3:
        [[0, 3, 6],
         [1, 4, 7],
         [2, 5]]
        9x3:
        [[0, 3, 6],
         [1, 4, 7],
         [2, 5, 8]]
        10x3:
        [[0, 4, 8],
         [1, 5, 9],
         [2, 6],
         [3, 7]]

        # Note that this filter does not guarantee that `n` columns will be
        # present:
        >>> pprint(columns(range(4), 3), width=10)
        [[0, 2],
         [1, 3]]
    """
    try:
        n = int(n)
        thelist = list(thelist)
    except (ValueError, TypeError):
        return [thelist]
    list_len = len(thelist)
    split = list_len // n
    if list_len % n != 0:
        split += 1
    return [thelist[i::split] for i in range(split)]
    
def columns(thelist, n):
    """ Break a list into n peices, but "horizontally." That is, 
    columns_distributed(range(10), 3) gives::

            [[0, 1, 2],
             [3, 4, 5],
             [6, 7, 8],
             [9]]

        Clear as mud?
    """
    from math import ceil
    try:
        n = int(n)
        thelist = list(thelist)
    except (ValueError, TypeError):
        return [thelist]
    newlists = [list() for i in range(int(ceil(len(thelist) / float(n))))]
    for i, val in enumerate(thelist):
        newlists[i/n].append(val)
    return newlists

register.filter(rows)
register.filter(rows_distributed)
register.filter(columns)
register.filter(columns_distributed)

def _test():
    import doctest
    doctest.testmod()

if __name__ == "__main__":
    _test()


# -*- coding: utf-8 -*-
# Generated by Django 1.9.2 on 2016-03-02 11:37
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('hurumap', '0002_geography_long_name'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='geography',
            name='osm_area_id',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.2 on 2016-03-02 14:45
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('hurumap', '0003_remove_geography_osm_area_id'),
    ]

    operations = [
        migrations.AlterField(
            model_name='geography',
            name='name',
            field=models.CharField(db_index=True, max_length=100),
        ),
    ]



from hurumap.tests.support import HurumapTestCase
from hurumap.data.utils import get_stat_data
from hurumap.data.tables import FieldTable


class UtilsTestCase(HurumapTestCase):
    def test_get_stat_data_percentage(self):
        self.field_table(['gender'], """
lev,code,Male,10
lev,code,Female,20
""")
        data, total = get_stat_data(['gender'], 'lev', 'code', self.s)
        self.assertEqual(total, 30)
        self.assertEqual(data['Male']['numerators']['this'], 10)
        self.assertEqual(data['Male']['values']['this'], 33.33)
        self.assertEqual(data['Female']['numerators']['this'], 20)
        self.assertEqual(data['Female']['values']['this'], 66.67)

        data, total = get_stat_data(['gender'], 'lev', 'code', self.s, percent=False)
        self.assertEqual(total, 30)
        self.assertIsNone(data['Male']['numerators']['this'])
        self.assertEqual(data['Male']['values']['this'], 10)
        self.assertIsNone(data['Female']['numerators']['this'])
        self.assertEqual(data['Female']['values']['this'], 20)

    def test_get_stat_data_nulls(self):
        self.field_table(['gender'], """
lev,code,Male,10
lev,code,Female,
""")
        data, total = get_stat_data(['gender'], 'lev', 'code', self.s)
        self.assertEqual(total, 10)
        self.assertEqual(data['Male']['numerators']['this'], 10)
        self.assertEqual(data['Male']['values']['this'], 100)
        self.assertIsNone(data['Female']['numerators']['this'])
        self.assertIsNone(data['Female']['values']['this'])

        data, total = get_stat_data(['gender'], 'lev', 'code', self.s, percent=False)
        self.assertEqual(total, 10)
        self.assertIsNone(data['Male']['numerators']['this'])
        self.assertEqual(data['Male']['values']['this'], 10)
        self.assertIsNone(data['Female']['numerators']['this'])
        self.assertIsNone(data['Female']['values']['this'])

    def test_get_stat_data_nulls_with_denominator_key(self):
        table = FieldTable(['household goods'], universe='Households', denominator_key='total households')
        self.load_data(table, """
lev,code,fridge,10
lev,code,computer,5
lev,code,total households,
""")
        data, total = get_stat_data(['household goods'], 'lev', 'code', self.s)
        self.assertEqual(total, None)
        self.assertEqual(data['Fridge']['numerators']['this'], 10)
        self.assertIsNone(data['Fridge']['values']['this'])
        self.assertEqual(data['Computer']['numerators']['this'], 5)
        self.assertIsNone(data['Computer']['values']['this'])



from collections import OrderedDict
import itertools

from django.db import models
from django.utils.text import slugify


# Geographies
class GeoMixin(object):
    def as_dict(self):
        return {
            'full_geoid': self.geoid,
            'full_name': self.full_name,  # profile views use this as a name
            'name': self.full_name,  # API views use this as a name
            'short_name': self.name,
            'geo_level': self.geo_level,
            'geo_code': self.geo_code,
            'child_level': self.child_level,
            'parent_geoid': self.parent_geoid,
            'square_kms': self.square_kms,
        }

    def as_dict_deep(self):
        return {
            'this': self.as_dict(),
            'parents': OrderedDict((p.geo_level, p.as_dict()) for p in self.ancestors()),
        }

    def children(self):
        """ Get all objects that are direct children of this object.
        """
        return self.__class__.objects\
            .filter(parent_level=self.geo_level,
                    parent_code=self.geo_code)\
            .all()

    def split_into(self, level):
        """ Walk down the level hierarchy from here and return
        all the objects that are of geo_level +level+ and descendents
        of this geography.
        """
        from hurumap.geo import geo_data
        levels = [level] + geo_data.geo_levels[level]['ancestors']

        candidates = self.children()
        kids = set()
        while candidates:
            kids.update(c for c in candidates if c.geo_level == level)
            candidates = list(itertools.chain(*[c.children() for c in candidates if c.geo_level in levels]))
        return list(kids)

    @property
    def full_name(self):
        if self.long_name:
            return self.long_name

        from hurumap.geo import geo_data

        names = [self.name]
        names += [a.name for a in self.ancestors() if a.geo_level != geo_data.root_level]
        return ', '.join(names)

    @property
    def geoid(self):
        return '-'.join([self.geo_level, self.geo_code])

    @property
    def parent_geoid(self):
        if self.parent_level and self.parent_code:
            return '%s-%s' % (self.parent_level, self.parent_code)
        return None

    @property
    def slug(self):
        return slugify(self.name)

    @property
    def child_level(self):
        # official child level
        from hurumap.geo import geo_data
        kids = geo_data.geo_levels[self.geo_level]['children']
        return kids[0] if kids else None

    def __unicode__(self):
        return self.full_name


class GeographyBase(models.Model, GeoMixin):
    #: The level for this geography (eg. `country`) which, together with
    #: `geo_code`, makes up the unique geo id.
    geo_level = models.CharField(max_length=15, null=False)
    #: The code for this geography which must be unique for this level.
    #: Together with `geo_level`, this makes up the unique geo id.
    geo_code = models.CharField(max_length=10, null=False)

    #: Name of this geography.
    name = models.CharField(max_length=100, null=False, db_index=True)
    #: Long name of this geography, giving it context (such as a city or province)
    #: If this is null, it is computed based on the place's ancestors.
    long_name = models.CharField(max_length=100, null=True, db_index=True)
    #: Year when this geography was defined. (advanced).
    year = models.IntegerField(db_index=True, null=True)

    #: Area in square kilometers. Optional.
    square_kms = models.FloatField(null=True)

    # hierarchy
    #: The level of this geography's parent, or `None` if this is the root
    #: geography that has no parent.
    parent_level = models.CharField(max_length=15, null=True)
    #: The code of this geography's parent, or `None` if this is the root
    #: geography that has no parent.
    parent_code = models.CharField(max_length=10, null=True)

    class Meta:
        abstract = True
        unique_together = ('geo_level', 'geo_code')

    @property
    def parent(self):
        """ The parent of this geograhy, or `None` if this is the root of
        the hierarchy.
        """
        if not hasattr(self, '_parent'):
            if self.parent_level and self.parent_code:
                self._parent = self.__class__.objects.filter(geo_level=self.parent_level, geo_code=self.parent_code).first()
            else:
                self._parent = None

        return self._parent

    def ancestors(self):
        """ A list of the ancestors of this geography, all the way up to the root.
        This is an empty list if this geography is the root of the hierarchy.
        """
        ancestors = []
        g = self.parent
        while g:
            ancestors.append(g)
            g = g.parent
        return ancestors


class Geography(GeographyBase):
    pass



from django import template

register = template.Library()


@register.inclusion_tag('profile/_blocks/_comparative_list_item.html')
def build_comparative_item(sumlev, stat, stat_type, geography, decimals=None):
    if sumlev == 'CBSA':
        place_name = 'the %s' % geography['parents'][sumlev]['full_name']
    else:
        place_name = geography['parents'][sumlev]['short_name']

    item_context = {
        'place_name': place_name,
        'stat_type': stat_type,
        'decimals': decimals,
        'value': stat['values'][sumlev],
        'index': stat['index'][sumlev],
        'error': stat['error'][sumlev],
        'error_ratio': stat['error_ratio'][sumlev],
        'numerator': stat['numerators'][sumlev],
        'numerator_error': stat['numerator_errors'][sumlev],
    }
    return item_context



from sqlalchemy.ext.declarative import declarative_base, declared_attr
from hurumap.data.utils import _metadata


class Base(object):
    @declared_attr
    def __tablename__(cls):
        return cls.__name__.lower()

    def __repr__(self):
        return '%s(%s)' % (self.__class__.__name__,
                           ', '.join(['%s="%s"' % (c.name, getattr(self, c.name))
                                      for c in self.__table__.columns]))


Base = declarative_base(cls=Base, metadata=_metadata)



from django.core.management.base import BaseCommand
from census.models import Table

SUBJECT_AREA_TO_TOPICS = {
    'Age-Sex': 'age, gender',
    'Hispanic Origin': 'race',
    'Race': 'race',

    'Earnings': 'income',
    'Employment Status': 'employment',
    'Health Insurance': 'health insurance',
    'Income': 'income',
    'Industry-Occupation-Class of Worker': 'employment',
    'Journey to Work': 'employment, commute',
    'Poverty': 'poverty',
    'Transfer Programs': 'public assistance',

    'Ancestry': 'ancestry',
    'Children - Relationship': 'children',
    'Disability': 'disability',
    'Educational Attainment': 'education',
    'Fertility': 'fertility',
    'Foreign Birth': 'place of birth',
    'Grand(Persons) - Age of HH Members': 'children, grandparents',
    'Households - Families': 'families',
    'Language': 'language',
    'Marital Status': 'marital status',
    'Place of Birth - Native': 'place of birth',
    'Residence Last Year - Migration': 'migration',
    'School Enrollment': 'education',
    'Veteran Status': 'veterans',

    'Housing': 'housing',
}

TABLE_NAME_TEXT_TO_TOPICS = {
    'children': 'children',
    'disability': 'disability',
    'bachelor\'s degree': 'education',
    'education': 'education',
    'school': 'education',
    'employ': 'employment',
    'occupation': 'employment',
    'work': 'employment',
    'families': 'families',
    'family': 'families',
    'nonfamily': 'roommates',
    'grandparent': 'grandparents',
    'health insurance': 'health insurance',
    'living arrange': 'housing',
    #'household': 'households',
    'earnings': 'income',
    'income': 'income',
    'geographical mobility': 'migration',
    'poverty': 'poverty',
    'food stamps': 'public assistance',
    'public assistance': 'public assistance',
    '65 years and over': 'seniors',
    'transportation': 'commute',
    'va health care': 'veterans',
    'veteran': 'veterans',
}

TABLE_NAME_TEXT_TO_FACETS = {
    'by age': 'age',
    'age by': 'age',
    'citizenship': 'citizenship',
    'naturalization': 'citizenship',
    'by famil': 'families',
    'by sex': 'gender',
    'sex by': 'gender',
    #'by household': 'household type',
    #'household type by': 'household type',
    'language': 'language',
    'marriage': 'marital status',
    'marital': 'marital status',
    'nativity': 'place of birth',
    'place of birth': 'place of birth',
    #'by relationship': 'relationship type',
    '(white': 'race',
    '(black': 'race',
    '(american': 'race',
    '(asian': 'race',
    '(native': 'race',
    '(some other race': 'race',
    '(two or more races': 'race',
    'hispanic': 'race',
}

class Command(BaseCommand):
    help = 'Uses dicts above to apply taxonomy to Census table names.'
    def handle(self, *args, **options):
        table_list = Table.objects.all()
        
        for table in table_list:
            table_name = table.table_name
            subject_area = table.subject_area
            topics = []
            print table_name

            if subject_area in SUBJECT_AREA_TO_TOPICS:
                # only keep the rows in the subject areas we want
                topics.extend(
                    [topic.strip() for topic in SUBJECT_AREA_TO_TOPICS[subject_area].split(',')]
                )
                
                for key in TABLE_NAME_TEXT_TO_TOPICS:
                    if key in table_name.lower():
                        topics.extend(
                            [topic.strip() for topic in TABLE_NAME_TEXT_TO_TOPICS[key].split(',')]
                        )
                
                # for now, we're storing these in the same place
                for key in TABLE_NAME_TEXT_TO_FACETS:
                    if key in table_name.lower():
                        topics.extend(
                            [facet.strip() for facet in TABLE_NAME_TEXT_TO_FACETS[key].split(',')]
                        )
                
                topics = ', '.join(sorted(set(topics)))
                table.topics = topics
                table.save()
                


import os.path
import json
import logging
from itertools import chain

from django.conf import settings
from django.utils.module_loading import import_string
from django.db.models import Q
from django.contrib.staticfiles.storage import staticfiles_storage

from hurumap.data.utils import LocationNotFound
from hurumap.models import Geography

log = logging.getLogger(__name__)


# GDAL is difficult to install, so we make it an optional dependency.
# Here, we check if it's installed and warn if it isn't.
try:
    import osgeo.gdal  # noqa
    HAS_GDAL = True
except ImportError:
    HAS_GDAL = False


class GeoData(object):
    """ General hurumap geography helper object.

    This object helps HURUmap load geographies, navigate geo level hierarchies,
    find locations, etc. It's a good place to override this functionality
    if you want to use a different geometry setup.

    To override behaviour, implement your own GeoData object (probably inheriting
    from this one), then set the `HURUMAP['geodata']` to the dotted path of your
    new class in your `settings.py`. HURUmap will then load that class and make
    it available as `hurumap.geo.geo_data`.
    """
    def __init__(self):
        self.geo_model = Geography
        self.setup_levels()
        self.setup_geometry()

    def setup_levels(self):
        """ Setup the summary level hierarchy from the `HURUMAP['levels']` and
        `HURUMAP['comparative_levels']` settings.
        """
        self.comparative_levels = ['this'] + settings.HURUMAP['comparative_levels']
        self.geo_levels = settings.HURUMAP['levels']

        parents = {}
        for code, level in self.geo_levels.iteritems():
            level.setdefault('name', code)
            level.setdefault('plural', code + 's')
            level.setdefault('children', [])
            level['sumlev'] = code

            for kid in level['children']:
                parents.setdefault(kid, []).append(code)

        # fold in the ancestors
        def climb(code):
            return chain(parents.get(code, []), *[climb(p) for p in parents.get(code, [])])

        for code, items in parents.iteritems():
            self.geo_levels[code]['ancestors'] = list(set(climb(code)))

        # root level
        roots = [key for key, lev in self.geo_levels.iteritems() if not lev.get('ancestors')]
        if not roots or len(roots) > 1:
            raise ValueError("geo_levels must have a single root item, but we found: %s" % roots)
        self.root_level = roots[0]

    def setup_geometry(self):
        """ Load boundaries from geojson shape files.
        """
        # map from levels to a dict of geoid-keyed feature
        # objects, including their geometry as shapely shapes
        #
        # eg.
        #
        # {
        #    'province': {
        #      'GT': {
        #        'properties': { ... },
        #        'shape': <shapely shape>
        #      }
        #    }
        # }
        #
        self.geometry = {}
        self.geometry_files = settings.HURUMAP.get('geometry_data', {})

        for level in self.geo_levels.iterkeys():
            fname, js = self.load_geojson_for_level(level)
            if not js:
                continue

            if js['type'] != 'FeatureCollection':
                raise ValueError("GeoJSON files must contain a FeatureCollection. The file %s has type %s" % (fname, js['type']))

            level_detail = self.geometry.setdefault(level, {})

            for feature in js['features']:
                props = feature['properties']
                shape = None

                if HAS_GDAL and feature['geometry']:
                    from shapely.geometry import asShape
                    try:
                        shape = asShape(feature['geometry'])
                    except ValueError as e:
                        log.error("Error parsing geometry for %s-%s from %s: %s. Feature: %s"
                                  % (level, props['code'], fname, e.message, feature), exc_info=e)
                        raise e

                level_detail[props['code']] = {
                    'properties': props,
                    'shape': shape
                }

    def load_geojson_for_level(self, level):
        fname = self.geometry_files.get(level, self.geometry_files.get(''))
        if not fname:
            return None, None

        # we have to have geojson
        name, ext = os.path.splitext(fname)
        if ext != '.geojson':
            fname = name + '.geojson'

        fname = staticfiles_storage.path(fname)

        # try load it
        try:
            with open(fname, 'r') as f:
                return fname, json.load(f)
        except IOError as e:
            if e.errno == 2:
                log.warn("Couldn't open geometry file %s -- no geometry will be available for level %s" % (fname, level))
            else:
                raise e

        return None, None

    def root_geography(self):
        """ First geography with no parents. """
        return self.geo_model.objects.filter(parent_level=None, parent_code=None, geo_level=self.root_level).first()

    def get_geography(self, geo_code, geo_level):
        """
        Get a geography object for this geography, or
        raise LocationNotFound if it doesn't exist.
        """
        geo = self.geo_model.objects.filter(geo_level=geo_level, geo_code=geo_code).first()
        if not geo:
            raise LocationNotFound('Invalid level and code: %s-%s' % (geo_level, geo_code))
        return geo

    def get_geometry(self, geo_level, geo_code):
        """ Get the geometry description for a geography. This is a dict
        with two keys, 'properties' which is a dict of properties,
        and 'shape' which is a shapely shape (may be None).
        """
        return self.geometry.get(geo_level, {}).get(geo_code)

    def get_locations(self, search_term, levels=None, year=None):
        """
        Try to find locations based on a search term, possibly limited
        to +levels+.

        Returns an ordered list of geo models.
        """
        search_term = search_term.strip()

        query = self.geo_model.objects\
            .filter(Q(name__icontains=search_term) |
                    Q(geo_code=search_term.upper()))\

        if levels:
            query = query.filter(geo_level__in=levels)

        if year is not None:
            query = query.filter(year=year)

        # TODO: order by level?
        objects = sorted(query[:10], key=lambda o: [o.geo_level, o.name, o.geo_code])
        return objects

    def get_locations_from_coords(self, longitude, latitude, levels=None):
        """
        Returns a list of geographies containing this point.
        """
        if not HAS_GDAL:
            gdal_missing(critical=True)

        from shapely.geometry import Point
        p = Point(float(longitude), float(latitude))
        geos = []

        for features in self.geometry.itervalues():
            for feature in features.itervalues():
                if feature['shape'] and feature['shape'].contains(p):
                    geo = self.get_geography(feature['properties']['code'],
                                             feature['properties']['level'])
                    if not levels or geo.geo_level in levels:
                        geos.append(geo)
        return geos

    def get_summary_geo_info(self, geo_code=None, geo_level=None):
        """ Get a list of (level, code) tuples of geographies that
        this geography should be compared against.

        This is the intersection of +comparative_levels+ and the
        ancestors of the geography.
        """
        geo = self.get_geography(geo_code, geo_level)
        ancestors = {g.geo_level: g for g in geo.ancestors()}

        return [(lev, ancestors[lev].geo_code) for lev in self.comparative_levels if lev in ancestors]

    def first_child_level(self):
        # first child level in the hierarchy
        return self.geo_levels[self.root_level]['children'][0]


geo_data = import_string(settings.HURUMAP['geodata'])()


def gdal_missing(critical=False):
    log.warn("NOTE: HURUmap is unable to load GDAL, it's probably not installed. "
             "Some functionality such as data downloads and geolocation won't work. This is ok in development, but "
             "is a problem in production. For more information on installing GDAL, see http://wazimap.readthedocs.io/en/latest/")

    if critical:
        raise StandardError("GDAL must be installed for this functionality to work.")



from setuptools import setup, find_packages
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the relevant file
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

with open(path.join(here, 'VERSION')) as f:
    version = f.read().strip()

install_requires = [
    'Django>=1.8.0,<1.10',
    'SQLAlchemy>=0.9.4',
    'boto>=2.27.0',
    'django-cors-headers>=1.1.0',
    'dj-database-url>=0.4.0',
    'django-pipeline>=1.6.4',
    'ecdsa>=0.11',
    'futures>=3.0.4',
    'paramiko>=1.12.3,<2',
    'psycopg2>=2.5.2',
    'requests>=1.2.0',
    'unicodecsv>=0.9.4',
    'whitenoise>=1.0.6',
    'wsgiref>=0.1.2',
]

setup(
    name='hurumap',

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version=version,

    description='A Django application for exploring census and other place-specific data',
    long_description=long_description,

    # The project's main homepage.
    url='https://github.com/CodeForAfrica/HURUmap',

    # Author details
    author='Code for Africa',
    author_email='support@codeforafrica.org',

    # Choose your license
    license='MIT',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 4 - Beta',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
    ],

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages(),

    include_package_data=True,

    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=install_requires,

    setup_requires=[
        "setuptools_git >= 1.0",
    ],

    # List additional groups of dependencies here (e.g. development
    # dependencies). You can install these using the following syntax,
    # for example:
    # $ pip install -e .[dev,test]
    extras_require={
        'dev': ['nose', 'flake8'],
        'test': ['nose', 'flake8'],
        'gdal': ['GDAL==1.*', 'Shapely>=1.5.13'],
    },
)



from django.core.management.base import BaseCommand
from multiprocessing import Pool
from traceback import format_exc
from boto.s3.connection import S3Connection
from boto.s3.key import Key

import json
import cStringIO
import gzip

from ...profile import geo_profile, enhance_api_data

import logging
logging.basicConfig(level=logging.WARN)
logger = logging.getLogger(__name__)

s3 = S3Connection()

def s3_keyname(geoid):
    return '/1.0/data/profiles/%s.json' % geoid

def key(geoid):
    bucket = s3.get_bucket('embed.censusreporter.org')
    keyname = s3_keyname(geoid)
    key = Key(bucket, keyname)

    return key

def write_profile_json(s3_key, data):
    s3_key.metadata['Content-Type'] = 'application/json'
    s3_key.metadata['Content-Encoding'] = 'gzip'
    s3_key.storage_class = 'REDUCED_REDUNDANCY'

    # create gzipped version of json in memory
    memfile = cStringIO.StringIO()
    #memfile.write(data)
    with gzip.GzipFile(filename=s3_key.key, mode='wb', fileobj=memfile) as gzip_data:
        gzip_data.write(data)
    memfile.seek(0)

    # store static version on S3
    s3_key.set_contents_from_file(memfile)

def seed(geoid):
    logger.info("Working on {}".format(geoid))
    try:
        api_data = geo_profile(geoid)
        api_data = enhance_api_data(api_data)

        s3key = key(geoid)
        write_profile_json(s3key, json.dumps(api_data))
        logger.info("Wrote to key {}".format(s3key))
    except Exception, e:
        logger.error("Problem caching {}".format(geoid))
        logger.exception(e)
    logger.info("Done working on {}".format(geoid))


class Command(BaseCommand):
    help = 'Pre-generates some Census Reporter content and places it on S3.'

    def handle(self, *args, **options):
        if not args:
            print "Please include the name of a file containing the seed geo_ids."
            return False

        parallelism = 4
        if 'parallelism' in options:
            parallelism = int(options.get('parallelism'))

        pool = Pool(parallelism)

        seed_file = open(args[0], 'r')

        for geoid in seed_file:
            pool.apply_async(seed, (geoid.strip(),))

        pool.close()
        pool.join()



from django.db import models

class Table(models.Model):
    table_id = models.CharField(max_length=8)
    table_name = models.CharField(max_length=255)
    table_universe = models.CharField(max_length=128)
    table_size = models.IntegerField()
    subject_area = models.CharField(max_length=32)
    topics = models.CharField(max_length=255, blank=True)
    release = models.CharField(max_length=16)
    
    class Meta:
        ordering = ('release','table_id')

    def __unicode__(self):
        return '%s' % self.table_name

class Column(models.Model):
    table = models.ForeignKey(Table)
    parent_table_id = models.CharField(max_length=8)
    column_id = models.CharField(max_length=16)
    column_order = models.DecimalField(max_digits=4, decimal_places=1)
    column_name = models.CharField(max_length=255)
    indent_value = models.IntegerField(blank=True, null=True)
    parent_column_id = models.CharField(max_length=255)
    has_children = models.BooleanField()

    class Meta:
        ordering = ('table__table_id','table__release','column_id')

    def __unicode__(self):
        return '%s' % self.column_name


class SummaryLevel(models.Model):
    summary_level = models.CharField(max_length=3)
    name = models.CharField(max_length=128)
    slug = models.SlugField()
    short_name = models.CharField(max_length=128, blank=True)
    plural_name = models.CharField(max_length=128, blank=True)
    
    description = models.TextField(blank=True)
    census_description = models.TextField(blank=True)
    census_code_description = models.TextField(blank=True)
    census_notes = models.TextField(blank=True)
    source = models.CharField(max_length=64, blank=True)
    
    # Relationships
    parent = models.ForeignKey('self', related_name='children', blank=True, null=True)
    ancestors = models.ManyToManyField('self', related_name='descendants', symmetrical=False, blank=True)
    
    class Meta:
        ordering = ('summary_level',)

    def __unicode__(self):
        return '%s' % self.name
        
    @property
    def display_name(self):
        return self.short_name or self.name
        
    def pretty_ancestor_list(self):
        return ', '.join([ancestor.name for ancestor in self.ancestors.all()])

    def pretty_ancestor_options(self):
        _list = self.pretty_ancestor_list()
        return ' or '.join(_list.rsplit(',',1))

    def pretty_ancestor_sumlev_list(self):
        return ','.join([ancestor.summary_level for ancestor in self.ancestors.all()])

    def ancestor_sumlev_list(self):
        return [ancestor.summary_level for ancestor in self.ancestors.all()]


class SubjectConcept(models.Model):
    name = models.CharField(max_length=128)
    slug = models.SlugField()
    census_category = models.CharField(max_length=128, blank=True, default="Population")
    census_description = models.TextField(blank=True)
    census_history = models.TextField(blank=True)
    census_comparability = models.TextField(blank=True)
    census_notes = models.TextField(blank=True)
    description = models.TextField(blank=True)
    source = models.CharField(max_length=64, blank=True, default="American Community Survey Subject Definitions")
    
    class Meta:
        ordering = ('name',)
        
    def __unicode__(self):
        return '%s' % self.name
        
        
class Geography(models.Model):
    full_geoid = models.CharField(max_length=16)
    full_name = models.CharField(max_length=128)
    sumlev = models.CharField(max_length=3)
    geo_type = models.CharField(max_length=24)
    region = models.CharField(max_length=2, blank=True, null=True)
    region_name = models.CharField(max_length=24, blank=True)
    division = models.CharField(max_length=2, blank=True, null=True)
    division_name = models.CharField(max_length=24, blank=True)
    statefp = models.CharField(max_length=2, blank=True, null=True)
    geoid = models.CharField(max_length=24, blank=True, null=True)
    cd112fp = models.CharField(max_length=12, blank=True, null=True)
    cdsessn = models.CharField(max_length=12, blank=True, null=True)
    countyfp = models.CharField(max_length=12, blank=True, null=True)
    placefp = models.CharField(max_length=12, blank=True, null=True)
    classfp = models.CharField(max_length=12, blank=True)
    sldlst = models.CharField(max_length=12, blank=True, null=True)
    sldust = models.CharField(max_length=12, blank=True, null=True)
    elsdlea = models.CharField(max_length=12, blank=True, null=True)
    scsdlea = models.CharField(max_length=12, blank=True, null=True)
    unsdlea = models.CharField(max_length=12, blank=True, null=True)
    pcicbsa = models.CharField(max_length=1, blank=True, null=True)
    pcinecta = models.CharField(max_length=1, blank=True, null=True)
    csafp = models.CharField(max_length=12, blank=True, null=True)
    cbsafp = models.CharField(max_length=12, blank=True, null=True)
    metdivfp = models.CharField(max_length=12, blank=True, null=True)
    zcta5ce10 = models.CharField(max_length=12, blank=True, null=True)
    name = models.CharField(max_length=128, blank=True)
    namelsad = models.CharField(max_length=128, blank=True)
    lsad = models.CharField(max_length=4, blank=True, null=True)
    aland = models.CharField(max_length=24, blank=True, null=True)
    intptlat = models.CharField(max_length=16, blank=True)
    intptlon = models.CharField(max_length=16, blank=True)
    
    class Meta:
        ordering = ('full_geoid',)
        verbose_name_plural = "Geographies"

    def __unicode__(self):
        return '%s' % self.full_name
    
